﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using BioLab.Common;
using BioLab.ImageProcessing;
using System.Drawing;
using BioLab.GUI.Forms;
using System.Windows.Forms;
using BioLab.ImageProcessing.Topology;
using BioLab.DataStructures;
using System.ComponentModel;

namespace PRLab.FEI
{

  [AlgorithmInfo("Stretch del contrasto", Category = "FEI")]
  public class ContrastStretch : ImageOperation<Image<byte>, Image<byte>>
  {
    private int stretchDiscardPerc;

    [AlgorithmParameter]
    [DefaultValue(0)]
    public int StretchDiscardPercentage
    {
      get { return stretchDiscardPerc; }
      set
      {
        if (value < 0 || value > 100)
          throw new ArgumentOutOfRangeException("Inserire un valore fra 0 and 100.");
        stretchDiscardPerc = value;
      }
    }

    public ContrastStretch()
    {
    }

    public ContrastStretch(Image<byte> inputImage)
      : base(inputImage)
    {
      StretchDiscardPercentage = 0;
    }

    public ContrastStretch(Image<byte> inputImage, int stretchDiscard)
      : base(inputImage)
    {
      StretchDiscardPercentage = stretchDiscard;
    }

    public override void Run()
    {
      var hist = new BioLab.ImageProcessing.HistogramBuilder(InputImage).Execute();
      int nd = InputImage.PixelCount * StretchDiscardPercentage / 100;
      int i1 = 0;
      int sum = 0;
      while (sum < nd)
      {
        sum += hist[i1++];
      }
      int i2 = 255;
      sum = 0;
      while (sum < nd)
      {
        sum += hist[i2--];
      }
      int diff = i2 - i1;
      if (diff == 0)
        diff = 1;
      Result = new LookupTableTransform<byte>(InputImage,
                        p => ((p - i1) * 255 / diff).ClipToByte()).Execute();
    }
  }


  [AlgorithmInfo("Equalizzazione istogramma", Category = "FEI")]
  public class HistogramEqualization : ImageOperation<Image<byte>, Image<byte>>
  {
    public HistogramEqualization()
    {
    }

    public HistogramEqualization(Image<byte> inputImage)
      : base(inputImage)
    {
    }

    public override void Run()
    {
      var hist = new BioLab.ImageProcessing.HistogramBuilder(InputImage).Execute();
      for (int i = 1; i < 256; i++)
      {
        hist[i] += hist[i - 1];
      }
      Result = new LookupTableTransform<byte>(InputImage,
                        p => (hist[p] * 255 / InputImage.PixelCount).ClipToByte()).Execute();
    }
  }

  [AlgorithmInfo("Operazione aritmetica", Category = "FEI")]
  public class ImageArithmetic : ImageOperation<Image<byte>, Image<byte>, Image<byte>>
  {
    [AlgorithmParameter]
    [DefaultValue(defaultOperation)]
    public ImageArithmeticOperation Operation { get; set; }
    const ImageArithmeticOperation defaultOperation = ImageArithmeticOperation.Difference;

    public ImageArithmetic()
    {
    }

    public ImageArithmetic(Image<byte> image1, Image<byte> image2, ImageArithmeticOperation operation)
      : base(image1, image2)
    {
      Operation = operation;
    }

    public ImageArithmetic(Image<byte> image1, Image<byte> image2)
      : this(image1, image2, defaultOperation)
    {
    }

    public override void Run()
    {
      Result = new Image<byte>(InputImage1.Width, InputImage1.Height);
      for (int i = 0; i < InputImage1.PixelCount; i++)
      {
        switch (Operation)
        {
          case ImageArithmeticOperation.Add:
            Result[i] = (InputImage1[i] + InputImage2[i]).ClipToByte();
            break;
          case ImageArithmeticOperation.And:
            Result[i] = (InputImage1[i] & InputImage2[i]).ClipToByte();
            break;
          case ImageArithmeticOperation.Average:
            Result[i] = ((InputImage1[i] + InputImage2[i]) / 2).ClipToByte();
            break;
          case ImageArithmeticOperation.Darkest:
            Result[i] = Math.Min(InputImage1[i], InputImage2[i]);
            break;
          case ImageArithmeticOperation.Difference:
            Result[i] = (Math.Abs(InputImage1[i] - InputImage2[i])).ClipToByte();
            break;
          case ImageArithmeticOperation.Lightest:
            Result[i] = Math.Max(InputImage1[i], InputImage2[i]);
            break;
          case ImageArithmeticOperation.Or:
            Result[i] = (InputImage1[i] | InputImage2[i]).ClipToByte();
            break;
          case ImageArithmeticOperation.Subtract:
            Result[i] = (InputImage1[i] - InputImage2[i]).ClipToByte();
            break;
          case ImageArithmeticOperation.Xor:
            Result[i] = (InputImage1[i] ^ InputImage2[i]).ClipToByte();
            break;
          default:
            throw new ArgumentException();
        }
      }
    }
  }
}
